import { useRef, useEffect, createElement } from 'react';
import '../utils/useId.js';
import { unstable_useCreateElement } from '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-64f351ca.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import { a as useLiveRef } from '../chunk-6e18ef4b.js';
import '../chunk-459cc222.js';
import { a as warning } from '../chunk-e555b6ea.js';
import 'react-dom';
import { Portal } from '../Portal/Portal.js';
import '../Hidden/HiddenState.js';
import { useHidden } from '../Hidden/Hidden.js';
import { disableBodyScroll, enableBodyScroll } from 'body-scroll-lock';
import { useDialogState } from './DialogState.js';

function useDisclosureRef(hiddenId, shouldAssign) {
  var disclosureRef = useRef(null);
  useEffect(function () {
    if (!shouldAssign) return;
    disclosureRef.current = document.body.querySelector("[aria-controls~=\"" + hiddenId + "\"]");
  }, [hiddenId, shouldAssign]);
  return disclosureRef;
}

function usePreventBodyScroll(targetElementRef, shouldPrevent) {
  useEffect(function () {
    var element = targetElementRef.current;
    if (!element || !shouldPrevent) return undefined;
    disableBodyScroll(element);
    return function () {
      return enableBodyScroll(element);
    };
  }, [targetElementRef, shouldPrevent]);
}

var selector = "input, select, textarea, a[href], button, [tabindex], audio[controls], video[controls], [contenteditable]:not([contenteditable=false])";

function isHTMLElement(element) {
  return element instanceof HTMLElement;
}

function isDisabled(element) {
  return Boolean(element.disabled);
}

function hasTabIndex(element) {
  return element.hasAttribute("tabindex");
}

function hasNegativeTabIndex(element) {
  return hasTabIndex(element) && element.tabIndex < 0;
}

function isHidden(element) {
  if (element.parentElement && isHidden(element.parentElement)) return true;
  return element.hidden;
}

function isContentEditable(element) {
  var value = element.getAttribute("contenteditable");
  return value !== "false" && value != null;
}

function isFocusable(element) {
  if (!isHTMLElement(element)) return false;
  if (isHidden(element)) return false;
  if (isDisabled(element)) return false;
  var localName = element.localName;
  var focusableTags = ["input", "select", "textarea", "button"];
  if (focusableTags.indexOf(localName) >= 0) return true;
  var others = {
    a: function a() {
      return element.hasAttribute("href");
    },
    audio: function audio() {
      return element.hasAttribute("controls");
    },
    video: function video() {
      return element.hasAttribute("controls");
    }
  };

  if (localName in others) {
    return others[localName]();
  }

  if (isContentEditable(element)) return true;
  return hasTabIndex(element);
}
function isTabbable(element) {
  if (!isHTMLElement(element)) return false;
  if (!isFocusable(element)) return false;
  if (hasNegativeTabIndex(element)) return false;
  return true;
}
function getAllTabbableIn(container, fallbackToFocusable) {
  var allFocusable = Array.from(container.querySelectorAll(selector));
  var allTabbable = allFocusable.filter(isTabbable);

  if (!allTabbable.length && fallbackToFocusable) {
    return allFocusable;
  }

  return allTabbable;
}
function getFirstTabbableIn(container, fallbackToFocusable) {
  var _getAllTabbableIn = getAllTabbableIn(container, fallbackToFocusable),
      first = _getAllTabbableIn[0];

  return first || null;
}
function getLastTabbableIn(container, fallbackToFocusable) {
  var allTabbable = getAllTabbableIn(container, fallbackToFocusable);
  return allTabbable[allTabbable.length - 1] || null;
}

function hasNestedOpenDialogs(portal) {
  return portal.querySelectorAll("[data-dialog][aria-hidden=false]").length > 1;
}

function useFocusOnShow(dialogRef, portalRef, initialFocusRef, shouldFocus) {
  useEffect(function () {
    var dialog = dialogRef.current;
    var portal = portalRef.current; // If there're nested open dialogs, let them handle focus

    if (!shouldFocus || !dialog || !portal || hasNestedOpenDialogs(portal)) {
      return;
    }

    if (initialFocusRef && initialFocusRef.current) {
      initialFocusRef.current.focus();
    } else {
      var tabbable = getFirstTabbableIn(dialog, true);

      if (tabbable) {
        tabbable.focus();
      } else {
        dialog.focus();
      }
    }
  }, [dialogRef, initialFocusRef, shouldFocus]);
}

function usePortalRef(dialogRef, shouldAssign) {
  var portalRef = useRef(null);
  useEffect(function () {
    var dialog = dialogRef.current;
    if (!dialog || !shouldAssign) return;
    portalRef.current = dialog.closest(Portal.__selector);
  }, [dialogRef, shouldAssign]);
  return portalRef;
}

function useEventListenerOutside(targetRef, event, listener, shouldListen) {
  var listenerRef = useLiveRef(listener);
  useEffect(function () {
    if (!shouldListen) return undefined;

    var handleEvent = function handleEvent(e) {
      var element = targetRef.current;
      if (!element || element.contains(e.target)) return;
      listenerRef.current(e);
    };

    document.addEventListener(event, handleEvent, true);
    return function () {
      document.removeEventListener(event, handleEvent, true);
    };
  }, [targetRef, event, shouldListen]);
}

var attr = "data-invoke";
function useAttachAndInvoke(targetRef, containerRef, name, method, shouldInvoke) {
  // Attach
  useEffect(function () {
    var element = targetRef.current;
    if (!element || !method) return;
    Object.defineProperty(targetRef.current, name, {
      writable: true,
      value: method
    });
    var attribute = element.getAttribute(attr);
    var invokeValues = attribute ? attribute.split(" ") : [];

    if (invokeValues.indexOf(name) === -1) {
      element.setAttribute("data-invoke", [].concat(invokeValues, [name]).join(" "));
    }
  }, [targetRef, name, method]); // Invoke

  useEffect(function () {
    var container = containerRef.current;
    if (!container || !method || !shouldInvoke) return;
    var elements = container.querySelectorAll("[" + attr + "~=\"" + name + "\"]");
    elements.forEach(function (element) {
      return element[name]();
    });
  }, [containerRef, name, method, shouldInvoke]);
}

function hasNestedOpenModals(portal) {
  return portal.querySelectorAll("[role=dialog][aria-modal=true][aria-hidden=false]").length > 1;
}

function useFocusTrap(dialogRef, portalRef, shouldTrap) {
  var beforeElement = useRef(null);
  var afterElement = useRef(null); // Create before and after elements
  // https://github.com/w3c/aria-practices/issues/545

  useEffect(function () {
    var portal = portalRef.current;
    if (!portal || !shouldTrap) return undefined;

    if (!beforeElement.current) {
      beforeElement.current = document.createElement("div");
      beforeElement.current.tabIndex = 0;
      beforeElement.current.style.position = "fixed";
      beforeElement.current.setAttribute("aria-hidden", "true");
    }

    if (!afterElement.current) {
      afterElement.current = beforeElement.current.cloneNode();
    }

    portal.insertAdjacentElement("beforebegin", beforeElement.current);
    portal.insertAdjacentElement("afterend", afterElement.current);
    return function () {
      if (beforeElement.current) beforeElement.current.remove();
      if (afterElement.current) afterElement.current.remove();
    };
  }, [portalRef, shouldTrap]); // Focus trap

  useEffect(function () {
    var before = beforeElement.current;
    var after = afterElement.current;
    if (!shouldTrap || !before || !after) return undefined;

    var handleFocus = function handleFocus(e) {
      var portal = portalRef.current;
      var dialog = dialogRef.current;
      if (!portal || !dialog || hasNestedOpenModals(portal)) return;
      e.preventDefault();
      var isAfter = e.target === after;
      var tabbable = isAfter ? getFirstTabbableIn(dialog) : getLastTabbableIn(dialog);

      if (tabbable) {
        tabbable.focus();
      } else {
        // fallback to dialog
        dialog.focus();
      }
    };

    before.addEventListener("focus", handleFocus);
    after.addEventListener("focus", handleFocus);
    return function () {
      before.removeEventListener("focus", handleFocus);
      after.removeEventListener("focus", handleFocus);
    };
  }, [dialogRef, portalRef, shouldTrap]); // Click trap

  useEffect(function () {
    if (!shouldTrap) return undefined;

    var handleClick = function handleClick() {
      var dialog = dialogRef.current;
      var portal = portalRef.current;
      if (!dialog || !portal || hasNestedOpenModals(portal)) return;

      if (!portal.contains(document.activeElement)) {
        dialog.focus();
      }
    };

    document.addEventListener("click", handleClick);
    return function () {
      document.removeEventListener("click", handleClick);
    };
  }, [dialogRef, portalRef, shouldTrap]);
}

function useFocusOnHide(dialogRef, finalFocusRef, shouldFocus) {
  useEffect(function () {
    var dialog = dialogRef.current;
    if (!dialog || !shouldFocus) return;

    if (document.activeElement && !dialog.contains(document.activeElement) && isTabbable(document.activeElement)) {
      return;
    }

    if (finalFocusRef && finalFocusRef.current) {
      finalFocusRef.current.focus();
    }
  }, [dialogRef, finalFocusRef, shouldFocus]);
}

function useDialog(_ref, htmlProps) {
  if (htmlProps === void 0) {
    htmlProps = {};
  }

  var _ref$unstable_modal = _ref.unstable_modal,
      unstable_modal = _ref$unstable_modal === void 0 ? true : _ref$unstable_modal,
      _ref$unstable_hideOnE = _ref.unstable_hideOnEsc,
      unstable_hideOnEsc = _ref$unstable_hideOnE === void 0 ? true : _ref$unstable_hideOnE,
      _ref$unstable_hideOnC = _ref.unstable_hideOnClickOutside,
      unstable_hideOnClickOutside = _ref$unstable_hideOnC === void 0 ? true : _ref$unstable_hideOnC,
      _ref$unstable_prevent = _ref.unstable_preventBodyScroll,
      unstable_preventBodyScroll = _ref$unstable_prevent === void 0 ? true : _ref$unstable_prevent,
      _ref$unstable_autoFoc = _ref.unstable_autoFocusOnShow,
      unstable_autoFocusOnShow = _ref$unstable_autoFoc === void 0 ? true : _ref$unstable_autoFoc,
      _ref$unstable_autoFoc2 = _ref.unstable_autoFocusOnHide,
      unstable_autoFocusOnHide = _ref$unstable_autoFoc2 === void 0 ? true : _ref$unstable_autoFoc2,
      options = _objectWithoutPropertiesLoose(_ref, ["unstable_modal", "unstable_hideOnEsc", "unstable_hideOnClickOutside", "unstable_preventBodyScroll", "unstable_autoFocusOnShow", "unstable_autoFocusOnHide"]);

  var _options = _objectSpread({
    unstable_modal: unstable_modal,
    unstable_hideOnEsc: unstable_hideOnEsc,
    unstable_hideOnClickOutside: unstable_hideOnClickOutside,
    unstable_preventBodyScroll: unstable_preventBodyScroll,
    unstable_autoFocusOnShow: unstable_autoFocusOnShow,
    unstable_autoFocusOnHide: unstable_autoFocusOnHide
  }, options);

  _options = unstable_useOptions("useDialog", _options, htmlProps);
  var dialog = useRef(null);
  var portal = usePortalRef(dialog, _options.visible);
  var disclosure = useDisclosureRef(_options.unstable_hiddenId, _options.visible);
  unstable_preventBodyScroll = !unstable_modal ? false : unstable_preventBodyScroll;
  usePreventBodyScroll(dialog, _options.visible && unstable_preventBodyScroll);
  useFocusTrap(dialog, portal, _options.visible && unstable_modal);
  useFocusOnShow(dialog, portal, _options.unstable_initialFocusRef, _options.visible && unstable_autoFocusOnShow);
  useFocusOnHide(dialog, _options.unstable_finalFocusRef || disclosure, !_options.visible && unstable_autoFocusOnHide); // Close all nested dialogs when parent dialog closes

  useAttachAndInvoke(dialog, portal, "hide", _options.hide, !_options.visible);

  var hide = function hide(e) {
    // Ignore disclosure since a click on it will already close the dialog
    if (e.target !== disclosure.current && _options.hide) {
      _options.hide();
    }
  }; // Hide on click outside


  useEventListenerOutside( // Portal, not dialog, so clicks on nested dialogs/portals don't close
  portal, "click", hide, _options.visible && unstable_hideOnClickOutside); // Hide on focus outside

  useEventListenerOutside(portal, "focus", hide, _options.visible && !unstable_modal && unstable_hideOnClickOutside);
  htmlProps = mergeProps({
    ref: dialog,
    role: "dialog",
    tabIndex: -1,
    "aria-modal": unstable_modal,
    "data-dialog": true,
    onKeyDown: function onKeyDown(event) {
      var keyMap = {
        Escape: function Escape() {
          if (!_options.hide || !unstable_hideOnEsc) return;
          event.stopPropagation();

          _options.hide();
        }
      };

      if (event.key in keyMap) {
        keyMap[event.key]();
      }
    }
  }, htmlProps);
  htmlProps = useHidden(_options, htmlProps);
  htmlProps = unstable_useProps("useDialog", _options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useHidden.__keys, useDialogState.__keys, ["unstable_modal", "unstable_hideOnEsc", "unstable_hideOnClickOutside", "unstable_preventBodyScroll", "unstable_initialFocusRef", "unstable_finalFocusRef", "unstable_autoFocusOnShow", "unstable_autoFocusOnHide"]);
useDialog.__keys = keys;
var Dialog = unstable_createComponent({
  as: "div",
  useHook: useDialog,
  useCreateElement: function useCreateElement(type, props, children) {
    process.env.NODE_ENV !== "production" ? warning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props.\nSee https://www.w3.org/TR/wai-aria-practices-1.1/#dialog_roles_states_props", "Dialog") : void 0;
    var element = unstable_useCreateElement(type, props, children);
    return createElement(Portal, null, element);
  }
});

export { Dialog, useDialog };
